//
//  SilentPlayerAppDelegate.m
//  SilentPlayer
//
//  Created by Craig Hockenberry on 4/18/11.
//  Copyright 2011 The Iconfactory. All rights reserved.
//

#import "SilentPlayerAppDelegate.h"

#import <Carbon/Carbon.h> // for AppleScript definitions


@interface SilentPlayerAppDelegate ()

@property (assign) PlayerState playerState;
@property (assign) NSInteger playerTrack;
@property (assign) NSInteger playerTimeRemaining;
@property (retain) NSTimer *playerTimer;

@end

@implementation SilentPlayerAppDelegate

@synthesize playerState, playerTrack, playerTimeRemaining, playerTimer;

@synthesize window;

@synthesize artworkImageView;
@synthesize artistTextField, trackTextField, albumTextField;
@synthesize timeRemainingLabel, timeRemainingTextField;
@synthesize previousButton, nextButton, playPauseButton;

typedef struct _SongData
{
	NSString *artist;
	NSString *track;
	NSString *album;
	NSString *artwork;
	NSInteger length;
} SongData;

const NSInteger songCount = 4;
static SongData songs[songCount] =
{
	// Artist, Track, Album, Artwork, Length (seconds)
	{ @"Simon & Garfunkel", @"The Sound of Silence", @"Sounds of Silence", @"simon_and_garfunkel.jpg", 183 },
	{ @"John Cage", @"4'33\"", @"Classical Hits", @"cage.jpg", 301 },
	{ @"Elvis Presley", @"Silent Night", @"Elvis' Christmas Album", @"elvis.jpg", 143 },
	{ @"Smashing Pumkpins", @"Quiet", @"Siamese Dream", @"smashing_pumpkins.jpg", 223 }
};

- (void)updateWindowWithPlayerStateAndStartingTrack:(BOOL)startTrack
{
	if (playerState == PlayerStateStopped) {
		[artworkImageView setHidden:YES];
		
		[artistTextField setStringValue:@""];
		[trackTextField setStringValue:@"Stopped"];
		[albumTextField setStringValue:@""];
		
		[timeRemainingLabel setHidden:YES];
		[timeRemainingTextField setHidden:YES];
		
		[previousButton setEnabled:NO];
		[nextButton setEnabled:NO];
		
		[playPauseButton setTitle:@"►"];
	}
	else {
		[artworkImageView setHidden:NO];
		[artworkImageView setImage:[NSImage imageNamed:songs[playerTrack].artwork]];

		[artistTextField setStringValue:songs[playerTrack].artist];
		[trackTextField setStringValue:songs[playerTrack].track];
		[albumTextField setStringValue:songs[playerTrack].album];

		if (startTrack) {
			self.playerTimeRemaining = songs[playerTrack].length;
		}
		NSInteger minutesRemaining = playerTimeRemaining / 60;
		NSInteger secondsRemaining = playerTimeRemaining - (minutesRemaining * 60);
		NSString *timeRemaining = [NSString stringWithFormat:@"%d:%02d", minutesRemaining, secondsRemaining];
		[timeRemainingTextField setStringValue:timeRemaining];

		[timeRemainingLabel setHidden:NO];
		[timeRemainingTextField setHidden:NO];

		[previousButton setEnabled:YES];
		[nextButton setEnabled:YES];
		
		if (playerState == PlayerStatePaused) {
			[playPauseButton setTitle:@"►"];
		}
		else {
			[playPauseButton setTitle:@"Ⅱ"];
		}
	}
}

- (void)postDistributedNotificationForStateChange
{
	// NOTE: Posting a distributed notification allows Take Five to update its display and state to match the application. This will provide a
	// much better user experience because user can see what's changed (for example, the new track's name.)

	NSLog(@"%s called", __PRETTY_FUNCTION__);
	[[NSDistributedNotificationCenter defaultCenter] postNotificationName:@"com.iconfactory.SilentPlayer.stateChange" object:nil userInfo:nil deliverImmediately:YES];
}

- (void)playerTimerFired:(NSTimer *)timer
{
	if (playerState == PlayerStatePlaying) {
		self.playerTimeRemaining = playerTimeRemaining - 1;
		if (playerTimeRemaining < 0) {
			[self nextTrack:self];
		}
		else {
			[self updateWindowWithPlayerStateAndStartingTrack:NO];
		}
	}
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	self.playerState = PlayerStateStopped;
	self.playerTrack = 0;
	self.playerTimeRemaining = 0;
	self.playerTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(playerTimerFired:) userInfo:nil repeats:YES];

	[self updateWindowWithPlayerStateAndStartingTrack:NO];
}

- (BOOL)windowShouldClose:(id)sender
{
	[NSApp terminate:self];
	return YES;
}

- (IBAction)previousTrack:(id)sender
{
	self.playerTrack = playerTrack - 1;
	if (playerTrack < 0) {
		self.playerTrack = 0;
		self.playerState = PlayerStateStopped;
	}
	[self updateWindowWithPlayerStateAndStartingTrack:YES];
	[self postDistributedNotificationForStateChange];
}

- (IBAction)nextTrack:(id)sender
{
	self.playerTrack = playerTrack + 1;
	if (playerTrack >= songCount) {
		self.playerTrack = 0;
		self.playerState = PlayerStateStopped;
	}
	[self updateWindowWithPlayerStateAndStartingTrack:YES];
	[self postDistributedNotificationForStateChange];
}

- (IBAction)playPause:(id)sender
{
	if (playerState == PlayerStateStopped) {
		self.playerState = PlayerStatePlaying;
		[self updateWindowWithPlayerStateAndStartingTrack:YES];
	}
	else if (playerState == PlayerStatePaused) {
		self.playerState = PlayerStatePlaying;
		[self updateWindowWithPlayerStateAndStartingTrack:NO];
	}
	else {
		self.playerState = PlayerStatePaused;
		[self updateWindowWithPlayerStateAndStartingTrack:NO];
	}
	[self postDistributedNotificationForStateChange];
}

- (IBAction)pause:(id)sender
{
	self.playerState = PlayerStatePaused;
	[self updateWindowWithPlayerStateAndStartingTrack:NO];
	[self postDistributedNotificationForStateChange];
}


- (IBAction)stop:(id)sender
{
	self.playerState = PlayerStateStopped;
	[self updateWindowWithPlayerStateAndStartingTrack:NO];
	[self postDistributedNotificationForStateChange];
}

#pragma mark - AppleScript support

/*
	NOTE: Make sure that the following items are added to the app's Info.plist settings:

	<key>NSAppleScriptEnabled</key>
	<true/>
	<key>OSAScriptingDefinition</key>
	<string>AppleScriptSuite.sdef</string>

	This enables AppleScript for the application and gives Cocoa a reference to the scripting commands that are handled.

	For more information on adding AppleScript to your audio application, see: http://rogueamoeba.com/utm/2011/04/06/airfoil-integration/
*/

- (BOOL)application:(NSApplication*)sender delegateHandlesKey:(NSString*)key
{
	return [[NSSet setWithObjects: @"playerStateValue", @"artist", @"track", @"album", @"artwork", nil] containsObject:key];
}

- (NSString *)playerStateValue
{
	NSString *result = @"stopped";
	
	if (playerState == PlayerStatePaused) {
		result = @"paused";
	}
	else if (playerState == PlayerStatePlaying) {
		result = @"playing";
	}
	
	return result;
}

- (NSString *)artist
{
	return songs[playerTrack].artist;
}

- (NSString *)track
{
	return songs[playerTrack].track;
}

- (NSString *)album
{
	return songs[playerTrack].album;
}

- (NSData *)artwork
{
	NSImage *image = [NSImage imageNamed:songs[playerTrack].artwork];
	NSData *result = [image TIFFRepresentation];
	return result;
}

@end


@implementation NSData (ImageDataScriptAdditions)

// NOTE: these methods define the "image data" type used to return the artwork.

+ (id)scriptingImageDataWithDescriptor:(NSAppleEventDescriptor *)descriptor
{
	NSData *result = nil;
	
	if (! ([descriptor descriptorType] == typeType && [descriptor typeCodeValue] == cMissingValue)) {
		if ([descriptor descriptorType] != typeTIFF) {
			descriptor = [descriptor coerceToDescriptorType:typeTIFF];
		}
		if (descriptor != nil) {
			result = [descriptor data];
		}
	}
	
	return result;
}

- (id)scriptingImageDataDescriptor
{
	return [NSAppleEventDescriptor descriptorWithDescriptorType:typeTIFF data:self];
}

@end

// NOTE: these classes perform the commands defined in the AppleScript suite. In most cases, you'll just map them to the actions
// that are already hooked up in your user interface (like we're doing with the button actions below.)

@implementation PauseScriptCommand

- (id)performDefaultImplementation
{
	[[NSApp delegate] pause:self];
	return nil;
}

@end

@implementation PlayPauseScriptCommand

- (id)performDefaultImplementation
{
	[[NSApp delegate] playPause:self];
	return nil;
}

@end

@implementation NextScriptCommand

- (id)performDefaultImplementation
{
	[[NSApp delegate] nextTrack:self];
	return nil;
}

@end

@implementation PreviousScriptCommand

- (id)performDefaultImplementation
{
	[[NSApp delegate] previousTrack:self];
	return nil;
}

@end
